@@ -207,12 +207,12 @@ class OscOptions(BaseModel):
                 setattr(self, field_name, None)
                 return
 
-        if field.origin_type is Password:
+        if field.origin_type == Password:
             value = Password(value)
             setattr(self, field_name, value)
             return
 
-        if field.type is List[HttpHeader]:
+        if field.type == List[HttpHeader]:
             value = http.client.parse_headers(BytesIO(value.strip().encode("utf-8"))).items()
             setattr(self, field_name, value)
             return
@@ -1308,6 +1308,18 @@ class Options(OscOptions):
         ini_key="download-assets-cmd",
     )  # type: ignore[assignment]
 
+    queryrecipe_cmd: str = Field(
+        default=
+            shutil.which("queryrecipe", path="/usr/bin:/usr/lib/build:/usr/lib/obs-build")
+            or "/usr/lib/build/queryrecipe",
+        description=textwrap.dedent(
+            """
+            Path to the 'queryrecipe' tool.
+            """
+        ),
+        ini_key="queryrecipe-cmd",
+    )  # type: ignore[assignment]
+
     obs_scm_bridge_cmd: str = Field(
         default=
             shutil.which("obs_scm_bridge", path="/usr/lib/obs/service")
