@@ -3,11 +3,14 @@ from .exceptions import BranchDoesNotExist
 from .exceptions import BranchExists
 from .exceptions import ForkExists
 from .exceptions import GiteaException
+from .exceptions import RepoExists
+from .exceptions import response_to_exception
 from .branch import Branch
 from .conf import Config
 from .conf import Login
 from .fork import Fork
 from .git import Git
+from .json import json_dumps
 from .pr import PullRequest
 from .repo import Repo
 from .ssh_key import SSHKey
