@@ -1,8 +1,11 @@
 import copy
 import http.client
 import json
+import re
 import time
 import urllib.parse
+from typing import Dict
+from typing import Generator
 from typing import Optional
 
 import urllib3
@@ -12,6 +15,19 @@ import urllib3.response
 from .conf import Login
 
 
+RE_HTTP_HEADER_LINK = re.compile('<(?P<url>.*?)>; rel="(?P<rel>.*?)",?')
+
+
+def parse_http_header_link(link: str) -> Dict[str, str]:
+    """
+    Parse RFC8288 "link" http headers into {"rel": "url"}
+    """
+    result = {}
+    for match in RE_HTTP_HEADER_LINK.findall(link):
+        result[match[1]] = match[0]
+    return result
+
+
 class GiteaHTTPResponse:
     """
     A ``urllib3.response.HTTPResponse`` wrapper
@@ -49,7 +65,14 @@ class Connection:
         self.host = parsed_url.hostname
         assert self.host is not None
         self.port = alternative_port if alternative_port else parsed_url.port
-        self.conn = ConnectionClass(host=self.host, port=self.port)
+
+        conn_kwargs = {}
+
+        if urllib3.__version__.startswith("1."):
+            # workaround for urllib3 v1: TypeError: 'object' object cannot be interpreted as an integer
+            conn_kwargs["timeout"] = 60
+
+        self.conn = ConnectionClass(host=self.host, port=self.port, **conn_kwargs)
 
         # retries; variables are named according to urllib3
         self.retry_count = 3
@@ -89,9 +112,13 @@ class Connection:
         url_query_str = urllib.parse.urlencode(query, doseq=True)
         return urllib.parse.urlunsplit(("", "", url_path_str, url_query_str, ""))
 
-    def request(self, method, url, json_data: Optional[dict] = None) -> GiteaHTTPResponse:
+    def request(
+        self, method, url, json_data: Optional[dict] = None, *, context: Optional[dict] = None
+    ) -> GiteaHTTPResponse:
         """
         Make a request and return ``GiteaHTTPResponse``.
+
+        :param context: Additional parameters passed as **kwargs to an exception if raised
         """
         headers = {
             "Content-Type": "application/json",
@@ -127,18 +154,39 @@ class Connection:
             self.conn.close()
 
         if isinstance(response, http.client.HTTPResponse):
-            result = GiteaHTTPResponse(urllib3.response.HTTPResponse.from_httplib(response))
+            response = GiteaHTTPResponse(urllib3.response.HTTPResponse.from_httplib(response))
         else:
-            result = GiteaHTTPResponse(response)
+            response = GiteaHTTPResponse(response)
 
         if not hasattr(response, "status"):
             from .exceptions import GiteaException  # pylint: disable=import-outside-toplevel,cyclic-import
 
-            raise GiteaException(result)
+            raise GiteaException(response)
 
         if response.status // 100 != 2:
-            from .exceptions import GiteaException  # pylint: disable=import-outside-toplevel,cyclic-import
+            from .exceptions import response_to_exception
 
-            raise GiteaException(result)
+            raise response_to_exception(response, context=context)
 
-        return result
+        return response
+
+    def request_all_pages(
+        self, method, url, json_data: Optional[dict] = None, *, context: Optional[dict] = None
+    ) -> Generator[GiteaHTTPResponse, None, None]:
+        """
+        Make a request and yield ``GiteaHTTPResponse`` instances for each page.
+        Arguments are forwarded to the underlying ``request()`` call.
+        """
+
+        while True:
+            response = self.request(method, url, json_data=json_data, context=context)
+            yield response
+
+            if "link" not in response.headers:
+                break
+
+            links = parse_http_header_link(response.headers["link"])
+            if "next" in links:
+                url = links["next"]
+            else:
+                break
